// $Id: PagerWidget.js,v 1.1 2009/10/07 22:23:31 jpmckinney Exp $

(function($) {

/**
 * Heavily inspired by the Ruby on Rails will_paginate gem.
 * I choose not to implement Drupal's theme_pager().
 * @expects this.target to be a list.
 */
AjaxSolr.PagerWidget = AjaxSolr.AbstractWidget.extend({
  /**
   * How many links are shown around the current page.
   *
   * @field
   * @public
   */
  innerWindow: 4,

  /**
   * How many links are around the first and the last page.
   *
   * @field
   * @public
   */
  outerWindow: 1,

  /**
   * The previous page link label.
   *
   * @field
   * @public
   */
  prevLabel: '&laquo; previous',

  /**
   * The next page link label.
   *
   * @field
   * @public
   */
  nextLabel: 'next &raquo;',

  /**
   * Separator between pagination links.
   *
   * @field
   * @public
   */
  separator: '',

  /**
   * The current page number.
   *
   * @field
   * @private
   */
  currentPage: null,

  /**
   * The total number of pages.
   *
   * @field
   * @private
   */
  totalPages: null,

  /**
   * @return The gap in page links, which is represented by:
   *   <span class="pager-gap">&hellip;</span>
   */
  gapMarker: function() {
    return $('<span class="pager-gap">&hellip;</span>');
  },

  /**
   * @return The links for the visible page numbers.
   */
  windowedLinks: function() {
    var links = [];

    var prev = null;

    visible = this.visiblePageNumbers();
    for (var i = 0; i < visible.length; i++) {
      if (prev && visible[i] > prev + 1) links.push(this.gapMarker());
      links.push(this.pageLinkOrSpan(visible[i], [ 'pager-current' ]));
      prev = visible[i];
    }

    return links;
  },

  /**
   * @return The visible page numbers according to the window options.
   */ 
  visiblePageNumbers: function() {
    var windowFrom = this.currentPage - this.innerWindow;
    var windowTo = this.currentPage + this.innerWindow;

    // If the window is truncated on one side, make the other side longer
    if (windowTo > this.totalPages) {
      windowFrom = Math.max(0, windowFrom - (windowTo - this.totalPages));
      windowTo = this.totalPages;
    }
    if (windowFrom < 1) {
      windowTo = Math.min(this.totalPages, windowTo + (1 - windowFrom));
      windowFrom = 1;
    }

    var visible = [];

    // Always show the first page
    visible.push(1);
    // Don't add inner window pages twice
    for (var i = 2; i <= Math.min(1 + this.outerWindow, windowFrom - 1); i++) {
      visible.push(i);
    }
    // if the gap is just one page, close the gap
    if (1 + this.outerWindow == windowFrom - 2) {
      visible.push(windowFrom - 1);
    }
    // Don't add the first or last page twice
    for (var i = Math.max(2, windowFrom); i <= Math.min(windowTo, this.totalPages - 1); i++) {
      visible.push(i);
    }
    // if the gap is just one page, close the gap
    if (this.totalPages - this.outerWindow == windowTo + 2) {
      visible.push(windowTo + 1);
    }
    // Don't add inner window pages twice
    for (var i = Math.max(this.totalPages - this.outerWindow, windowTo + 1); i < this.totalPages; i++) {
      visible.push(i);
    }
    // Always show the last page, unless it's the first page
    if (this.totalPages > 1) {
      visible.push(this.totalPages);
    }

    return visible;
  },

  /**
   * @param page A page number.
   * @param classnames CSS classes to add to the page link.
   * @param text The inner HTML of the page link (optional).
   * @return The link or span for the given page.
   */
  pageLinkOrSpan: function(page, classnames, text) {
    text = text || page;

    if (page && page != this.currentPage) {
      return $('<a href="#"/>').html(text).attr('rel', this.relValue(page)).addClass(classnames[1]).click(this.clickHandler(page));
    }
    else {
      return $('<span/>').html(text).addClass(classnames.join(' '));
    }
  },

  /**
   * @param page A page number.
   * @return The click handler for the page link.
   */
  clickHandler: function(page) {
    var me = this;
    return function() {
      me.manager.doRequest((page - 1) * me.perPage);
      return false;
    }
  },

  /**
   * @param page A page number.
   * @return The <tt>rel</tt> attribute for the page link.
   */
  relValue: function(page) {
    switch (page) {
      case this.previousPage():
        return 'prev' + (page == 1 ? 'start' : '');
      case this.nextPage():
        return 'next';
      case 1:
        return 'start';
      default: 
        return '';
    }
  },

  /**
   * @return The page number of the previous page or null if no previous page.
   */
  previousPage: function() {
    return this.currentPage > 1 ? (this.currentPage - 1) : null;
  },

  /**
   * @return The page number of the next page or null if no next page.
   */
  nextPage: function() {
    return this.currentPage < this.totalPages ? (this.currentPage + 1) : null;
  },

  /**
   * Render the pagination header.
   *
   * @param perPage The number of items shown per results page.
   * @param offset The index in the result set of the first document to render.
   * @param total The total number of documents in the result set.
   */
  renderHeader: function(perPage, offset, total) {
    $(this.target).parent().find('h3').remove();
    if (this.totalPages) {
      $('<h3/>').text(Drupal.t('displaying @first to @last of @total', {
        '@first' : Math.min(total, offset + 1),
        '@last'  : Math.min(total, offset + perPage),
        '@total' : total
      })).insertBefore($(this.target));
    }
  },

  /**
   * Render the pagination links.
   *
   * @param links The links for the visible page numbers.
   */
  renderLinks: function(links) {
    $(this.target).empty();
    if (this.totalPages) {
      links.unshift(this.pageLinkOrSpan(this.previousPage(), [ 'pager-disabled', 'pager-prev' ], this.prevLabel));
      links.push(this.pageLinkOrSpan(this.nextPage(), [ 'pager-disabled', 'pager-next' ], this.nextLabel));
      AjaxSolr.theme('list_items', this.target, links, this.separator);
    }
  },

  handleResult: function(data) {
    var perPage = parseInt(data.responseHeader.params.rows);
    var offset = parseInt(data.responseHeader.params.start);
    var total = parseInt(data.response.numFound);

    // Normalize the offset to a multiple of perPage.
    offset = offset - offset % perPage;

    this.currentPage = Math.ceil((offset + 1) / perPage);
    this.totalPages = Math.ceil(total / perPage);
    this.perPage = perPage;

    this.renderHeader(perPage, offset, total);

    this.renderLinks(this.windowedLinks());
  }
});

})(jQuery);
